@@ -56,11 +56,13 @@ import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.SequenceInputStream;
+import java.net.URI;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Optional;
 import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeSet;
@@ -88,7 +90,7 @@ import com.amazonaws.services.s3.model.S3ObjectSummary;
 import com.amazonaws.services.s3.model.UploadPartRequest;
 import com.amazonaws.services.s3.model.UploadPartResult;
 import com.openexchange.exception.OXException;
-import com.openexchange.filestore.FileStorage;
+import com.openexchange.filestore.DestroyAwareFileStorage;
 import com.openexchange.filestore.FileStorageCodes;
 import com.openexchange.filestore.utils.TempFileHelper;
 import com.openexchange.java.Streams;
@@ -100,7 +102,7 @@ import com.openexchange.java.util.UUIDs;
  *
  * @author <a href="mailto:jan.bauerdick@open-xchange.com">Jan Bauerdick</a>
  */
-public class S3FileStorage implements FileStorage {
+public class S3FileStorage implements DestroyAwareFileStorage {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(S3FileStorage.class);
 
@@ -115,10 +117,12 @@ public class S3FileStorage implements FileStorage {
     private final String bucketName;
     private final String prefix;
     private final long chunkSize;
+    private final URI uri;
 
     /**
      * Initializes a new {@link S3FileStorage}.
      *
+     * @param uri The URI that fully qualifies this file storage
      * @param amazonS3 The underlying S3 client
      * @param clientSideEncryption Whether S3 client has client encryption enabled or not
      * @param serverSideEncryption Whether to use server side encryption or not
@@ -126,19 +130,30 @@ public class S3FileStorage implements FileStorage {
      * @param prefix The prefix to use; e.g. <code>"1337ctxstore"</code>
      * @param chunkSize The chunk size in bytes to use for multipart uploads
      */
-    public S3FileStorage(AmazonS3Client amazonS3, boolean clientSideEncryption, boolean serverSideEncryption, String bucketName, String prefix, long chunkSize) {
+    public S3FileStorage(URI uri, AmazonS3Client amazonS3, boolean clientSideEncryption, boolean serverSideEncryption, String bucketName, String prefix, long chunkSize) {
         super();
         BucketNameUtils.validateBucketName(bucketName);
         if (Strings.isEmpty(prefix) || prefix.contains(DELIMITER)) {
             throw new IllegalArgumentException(prefix);
         }
+        this.uri = uri;
         this.amazonS3 = amazonS3;
         this.clientSideEncryption = clientSideEncryption;
         this.serverSideEncryption = serverSideEncryption;
         this.bucketName = bucketName;
         this.prefix = prefix;
         this.chunkSize = chunkSize;
-        LOG.info("S3 file storage initialized for \"{}/{}{}\"", bucketName, prefix, DELIMITER);
+        LOG.debug("S3 file storage initialized for \"{}/{}{}\"", bucketName, prefix, DELIMITER);
+    }
+
+    @Override
+    public void onDestroyed() {
+        amazonS3.shutdown();
+    }
+
+    @Override
+    public URI getUri() {
+        return uri;
     }
 
     @Override
@@ -155,8 +170,9 @@ public class S3FileStorage implements FileStorage {
              * spool to file
              */
             if (!(input instanceof FileInputStream)) {
-                tmpFile = TempFileHelper.getInstance().newTempFile();
-                if (tmpFile != null) {
+                Optional<File> optionalTempFile = TempFileHelper.getInstance().newTempFile();
+                if (optionalTempFile.isPresent()) {
+                    tmpFile = optionalTempFile.get();
                     input = Streams.transferToFileAndCreateStream(input, tmpFile);
                 }
             }
@@ -260,7 +276,7 @@ public class S3FileStorage implements FileStorage {
 
     @Override
     public long getFileSize(final String name) throws OXException {
-        return getMetadata(addPrefix(name)).getContentLength();
+        return getContentLength(getMetadata(addPrefix(name)));
     }
 
     @Override
@@ -358,7 +374,7 @@ public class S3FileStorage implements FileStorage {
             SequenceInputStream inputStream = null;
             try {
                 s3Object = getObject(key);
-                long currentLength = s3Object.getObjectMetadata().getContentLength();
+                long currentLength = getContentLength(s3Object.getObjectMetadata());
                 if (currentLength != offset) {
                     throw FileStorageCodes.INVALID_OFFSET.create(Long.valueOf(offset), name, Long.valueOf(currentLength));
                 }
@@ -366,8 +382,9 @@ public class S3FileStorage implements FileStorage {
                  * spool to file
                  */
                 if (!(file instanceof FileInputStream)) {
-                    tmpFile = TempFileHelper.getInstance().newTempFile();
-                    if (tmpFile != null) {
+                    Optional<File> optionalTempFile = TempFileHelper.getInstance().newTempFile();
+                    if (optionalTempFile.isPresent()) {
+                        tmpFile = optionalTempFile.get();
                         file = Streams.transferToFileAndCreateStream(file, tmpFile);
                     }
                 }
@@ -385,12 +402,11 @@ public class S3FileStorage implements FileStorage {
             try {
                 String tempKey = addPrefix(tempName);
                 CopyObjectRequest copyObjectRequest = new CopyObjectRequest(bucketName, tempKey, bucketName, key);
-                ObjectMetadata metadata = new ObjectMetadata();
-                metadata = prepareMetadataForSSE(metadata);
+                ObjectMetadata metadata = prepareMetadataForSSE(getMetadata(tempKey).clone());
                 copyObjectRequest.setNewObjectMetadata(metadata);
                 amazonS3.copyObject(copyObjectRequest);
                 amazonS3.deleteObject(bucketName, tempKey);
-                return getMetadata(key).getContentLength();
+                return getContentLength(getMetadata(key));
             } catch (AmazonClientException e) {
                 throw wrap(e, key);
             }
@@ -415,8 +431,7 @@ public class S3FileStorage implements FileStorage {
         String tempKey = generateKey(true);
         try {
             CopyObjectRequest copyObjectRequest = new CopyObjectRequest(bucketName, key, bucketName, tempKey);
-            ObjectMetadata metadata = new ObjectMetadata();
-            metadata = prepareMetadataForSSE(metadata);
+            ObjectMetadata metadata = prepareMetadataForSSE(getMetadata(key).clone());
             copyObjectRequest.setNewObjectMetadata(metadata);
             amazonS3.copyObject(copyObjectRequest);
             /*
@@ -559,7 +574,7 @@ public class S3FileStorage implements FileStorage {
     }
 
     private ObjectMetadata prepareMetadataForSSE(ObjectMetadata metadata) {
-        if(serverSideEncryption) {
+        if (serverSideEncryption) {
             metadata.setSSEAlgorithm(ObjectMetadata.AES_256_SERVER_SIDE_ENCRYPTION);
         }
         return metadata;
@@ -594,4 +609,23 @@ public class S3FileStorage implements FileStorage {
             Streams.close(chunk);
         }
     }
+
+    /**
+     * Extracts the effective content length from the supplied S3 object metadata, which is the length of the unencrypted content if specified, or the plain content length, otherwise.
+     *
+     * @param metadata The metadata to get the content length from
+     * @return The length of the unencrypted content if specified, or the plain content length, otherwise
+     */
+    private long getContentLength(ObjectMetadata metadata) throws OXException {
+        String unencryptedContentLength = metadata.getUserMetaDataOf(com.amazonaws.services.s3.Headers.UNENCRYPTED_CONTENT_LENGTH);
+        if (Strings.isNotEmpty(unencryptedContentLength)) {
+            try {
+                return Long.parseLong(unencryptedContentLength);
+            } catch (NumberFormatException e) {
+                throw FileStorageCodes.NO_NUMBER.create(e);
+            }
+        }
+        return metadata.getContentLength();
+    }
+
 }
